% This script derives optical constants for the a-c plane monoclinic crystals from oriened reflectance data. 
% 
% The functions called by the main script are:
% disp_model_wrap_ac.m  :Wrapper function needed to ease use of
%                             lsqcurvefit.
% dispersion_model_ac.m :Actual dispersion model called by the
%                             wrapper function. Outputs optical constants 
%                             (n,k). 
% fresnel_ac.m          :Fresnel equations for non-normal incidence. Calculates 
%                             reflectance from optical constants. Incidence
%                             angle set to 30 degrees. Edit this funcion to
%                             change that.
% isint.m                    :Short boolean function that gives 'Yes' if
%                             the input is an integer. 
%
% inputs: param_values.txt - oscillator parameter estimates
% output: new_param_values.txt
%         optical_constants_ac.txt

%% Section 1
% read in reflectance data and trim to a range of wavenumbers
[v1,r_omega1]=textread('./Orth 0deg Wed Nov 09 12-36-20 2011 (GMT-05-00).CSV','%f %f');
[v2,r_omega2]=textread('./Orth 45 Wed Nov 09 12-47-58 2011 (GMT-05-00).CSV','%f %f');
[v3,r_omega3]=textread('./Orth 90 Wed Nov 09 13-08-04 2011 (GMT-05-00).CSV','%f %f');
[v4,r_omega4]=textread('./Orth 135 Wed Nov 09 13-19-36 2011 (GMT-05-00).CSV','%f %f');
[v5,rb]=textread('./bkgd sample holder Mon Apr 18 15-07-09 2011 (GMT-04-00).CSV','%f %f');

% Clip reflectance data to desired wavenumber range.
lo_wn=400; %set lower bound
up_wn=2000; %set upper bound

r_omega1=r_omega1(find(v1>lo_wn)); r_omega2=r_omega2(find(v2>lo_wn)); r_omega3=r_omega3(find(v3>lo_wn)); r_omega4=r_omega4(find(v4>lo_wn)); rb=rb(find(v4>lo_wn));
v1=v1(find(v1>lo_wn)); v2=v2(find(v2>lo_wn)); v3=v3(find(v3>lo_wn)); v4=v4(find(v4>lo_wn)); v5=v5(find(v5>lo_wn));

r_omega1=r_omega1(find(v1<up_wn)); r_omega2=r_omega2(find(v2<up_wn)); r_omega3=r_omega3(find(v3<up_wn)); r_omega4=r_omega4(find(v4<up_wn)); rb=rb(find(v5<up_wn));
v1=v1(find(v1<up_wn)); v2=v2(find(v2<up_wn)); v3=v3(find(v3<up_wn)); v4=v4(find(v4<up_wn)); v5=v5(find(v5<up_wn));

% Subtract backround spectrum of empty chamber with sample holder. Comment
% out the next line to skip this step.
r_omega1=r_omega1-rb; r_omega2=r_omega2-rb; r_omega1=r_omega1-rb; r_omega4=r_omega4-rb;

%Change wavenumber vector direction for each reflectance spectrum and
%combine into one array.
v1=v1(:); v2=v2(:); v3=v3(:); v4=v4(:); 
v=cat(1,v1,v2,v3,v4);

%Polarization angle with respect to the a-axis of the crystal for each reflectance
%spectrum.
omega_1=0; omega_2=45; omega_3=90; omega_4=135;
%extend omega into a vector with length of each reflectance spectrum and
%combine into one array
omega_1=repmat(omega_1,length(v1),1);
omega_2=repmat(omega_2,length(v2),1);
omega_3=repmat(omega_3,length(v3),1);
omega_4=repmat(omega_4,length(v4),1);
omega=cat(1,omega_1,omega_2,omega_3,omega_4);

% create xdata array for fitting the reflectance spectrum
% r=fn(v,omega)

p1=cat(2,v1,omega_1); p2=cat(2,v2,omega_2); p3=cat(2,v3,omega_3); p4=cat(2,v4,omega_4);
p=cat(2,v,omega);

% convert to reflectance from % reflectance 
r_omega1=0.01*r_omega1; r_omega2=0.01*r_omega2; r_omega3=0.01*r_omega3; r_omega4=0.01*r_omega4;
r=cat(1, r_omega1, r_omega2, r_omega3, r_omega4);

%% Section 2
% Read in parameter estimates, upper and lower bounds from 
% file param_values.txt

%get number of oscillators
fid=fopen('./param_values.txt');
ncols=length(textscan(fid,'','HeaderLines',1,'Delimiter',','));
disp(['number of oscillators: ', num2str(ncols)]);
fclose(fid);

%read paramter input file
parameter_file = fopen('./param_values.txt');
textformat=repmat('%f',1,ncols);
nu=textscan(parameter_file,textformat,'HeaderLines',2,'Delimiter',',');
gamm = textscan(parameter_file,textformat,'HeaderLines',1,'Delimiter',',');
Sk = textscan(parameter_file,textformat,'HeaderLines',1,'Delimiter',',');
phi = textscan(parameter_file,textformat,'HeaderLines',1,'Delimiter',',');
theta = textscan(parameter_file,textformat,'HeaderLines',1,'Delimiter',',');
textformat='%f';
epsilxx = textscan(parameter_file,textformat,'HeaderLines',1); 
epsilxy = textscan(parameter_file,textformat,'HeaderLines',1);
epsilyy = textscan(parameter_file,textformat,'HeaderLines',1);
epsilzz = textscan(parameter_file,textformat,'HeaderLines',1);
%read in lower bounds
textformat=repmat('%f',1,ncols);
lb_nu=textscan(parameter_file,textformat,'HeaderLines',2,'Delimiter',',');
lb_gamm = textscan(parameter_file,textformat,'HeaderLines',1,'Delimiter',',');
lb_Sk = textscan(parameter_file,textformat,'HeaderLines',1,'Delimiter',',');
lb_phi = textscan(parameter_file,textformat,'HeaderLines',1,'Delimiter',',');
lb_theta = textscan(parameter_file,textformat,'HeaderLines',1,'Delimiter',',');
textformat='%f';
lb_epsilxx = textscan(parameter_file,textformat,'HeaderLines',1); 
lb_epsilxy = textscan(parameter_file,textformat,'HeaderLines',1);
lb_epsilyy = textscan(parameter_file,textformat,'HeaderLines',1);
lb_epsilzz = textscan(parameter_file,textformat,'HeaderLines',1);
%read in upper bounds
textformat=repmat('%f',1,ncols);
ub_nu=textscan(parameter_file,textformat,'HeaderLines',2,'Delimiter',',');
ub_gamm = textscan(parameter_file,textformat,'HeaderLines',1,'Delimiter',',');
ub_Sk = textscan(parameter_file,textformat,'HeaderLines',1,'Delimiter',',');
ub_phi = textscan(parameter_file,textformat,'HeaderLines',1,'Delimiter',',');
ub_theta = textscan(parameter_file,textformat,'HeaderLines',1,'Delimiter',',');
textformat='%f';
ub_epsilxx = textscan(parameter_file,textformat,'HeaderLines',1); 
ub_epsilxy = textscan(parameter_file,textformat,'HeaderLines',1);
ub_epsilyy = textscan(parameter_file,textformat,'HeaderLines',1);
ub_epsilzz = textscan(parameter_file,textformat,'HeaderLines',1);
fclose(parameter_file);

% Concatenate together guesses to make coefficient vector. lsqcurvefit
% requires the model parameters to be in one vector or rectangular array.
coefg=cell2mat([nu,gamm,Sk,phi,theta,epsilxx,epsilxy,epsilyy,epsilzz]);

% Concatenate together upper and lower bounds.
lb=cell2mat([lb_nu,lb_gamm,lb_Sk,lb_phi,lb_theta,lb_epsilxx,lb_epsilxy,lb_epsilyy,lb_epsilzz]);
ub=cell2mat([ub_nu,ub_gamm,ub_Sk,ub_phi,ub_theta,ub_epsilxx,ub_epsilxy,ub_epsilyy,ub_epsilzz]);

% Allows the user to see if there is a mis-match between the number of
% parameters and the number of bounds
if(or(length(coefg)~=length(lb),length(coefg)~=length(ub)))
    display('Number of coefficients do not match number of bounds!');
end
disp(['number of coefficients: ', num2str(length(coefg))]); 
disp(['number of lower bounds: ', num2str(length(lb))]); 
disp(['number of upper bounds: ', num2str(length(ub))]);

%% Section 3
%fits

% Do nonlinear fit of the four reflectance spectra using lsqcurefit
options=optimset('Display','iter','MaxIter',100,'MaxFunEvals',50000); %First 2 options display fit iterations. 
                                                                     %This can be removed to save time. If 
                                                                     %fit doesn't converge try increasing
                                                                     %MaxIter
[coeff,resnorm,res,exitflag,output,lambda,J]=lsqcurvefit(@disp_model_wrap_ac,coefg,p,r,lb,ub,options); 

%% Section 4
% produce final model values, output text files of optical constants and
% final parameter valuies, and plot results

% run forward model to produce guess and fitted spectra emg and emf and
% create output text files of optical constants and final oscillator
% parameter values
[r_guess,n1g,n2g,k1g,k2g]=disp_model_wrap_ac(coefg,p);
L=length(v1);
r_omega1_guess=r_guess(1:L,1);
r_omega2_guess=r_guess(L+(1:L),1);
r_omega3_guess=r_guess(2.*L+(1:L),1);
r_omega4_guess=r_guess(3.*L+(1:L),1);
n1g=n1g(1:L); n2g=n2g(1:L);
k1g=k1g(1:L); k2g=k2g(1:L);

[r_final,n1,n2,k1,k2]=disp_model_wrap_ac(coeff,p);
r_omega1_final=r_final(1:L,1);
r_omega2_final=r_final(L+(1:L),1);
r_omega3_final=r_final(2.*L+(1:L),1);
r_omega4_final=r_final(3.*L+(1:L),1);
n1=n1(1:L); n2=n2(1:L);
k1=k1(1:L); k2=k2(1:L);

% calculate confidence bounds for reflectance fit
[rpred,del] = nlpredci(@disp_model_wrap_ac,p,coeff,res,'Jacobian',J+1e-6);
lower_r = rpred - del;
upper_r = rpred + del;

r_omega1_lower=lower_r(1:L,1);
r_omega2_lower=lower_r(L+(1:L),1);
r_omega3_lower=lower_r(2.*L+(1:L),1);
r_omega4_lower=lower_r(3.*L+(1:L),1);

r_omega1_upper=upper_r(1:L,1);
r_omega2_upper=upper_r(L+(1:L),1);
r_omega3_upper=upper_r(2.*L+(1:L),1);
r_omega4_upper=upper_r(3.*L+(1:L),1);

% write final oscillator parameters to file 'new_param_values.txt')
fid = fopen('param_values.txt','r');
i = 1;
tline = fgetl(fid);
temp_params{i} = tline;
while ischar(tline)
    i = i+1;
    tline = fgetl(fid);
    temp_params{i} = tline;
end
fclose(fid);
textformat=(repmat('%f,',1,ncols)); textformat=textformat(1:length(textformat)-1);
temp_params{3}=sprintf(textformat,coeff(1:ncols));
temp_params{5}=sprintf(textformat,coeff(ncols+(1:ncols)));
temp_params{7}=sprintf(textformat,coeff(2.*ncols+(1:ncols)));
temp_params{9}=sprintf(textformat,coeff(3.*ncols+(1:ncols)));
temp_params{11}=sprintf(textformat,coeff(4.*ncols+(1:ncols)));
temp_params{13}=sprintf('%f',coeff(5.*ncols+1));
temp_params{15}=sprintf('%f',coeff(5.*ncols+2));
temp_params{17}=sprintf('%f',coeff(5.*ncols+3));
temp_params{19}=sprintf('%f',coeff(5.*ncols+4));

fid = fopen('new_param_values.txt', 'w');
fprintf(fid, '%s\n', temp_params{:});
fclose(fid);

opconst=cat(2,v1,n1,n2,k1,k2);
dlmwrite('optical_constants_ac.txt', opconst, 'delimiter', '\t', 'precision', 6);

%% Section 5
%plot stuff

set(0,'DefaultAxesColorOrder',[0 0 1; .1 .8 .1; 1 0 0])

%plot modeled and observed reflectance spectra on the same axes
figure(1); clf;
subplot(4,1,1); plot(v1,r_omega1_final,v1,r_omega1,'LineWidth',3.0);
xlim([400 2000]);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('modeled','observed E || a',2)
subplot(4,1,2); plot(v2,r_omega2_final,v2,r_omega2,'LineWidth',3.0);
xlim([400 2000]);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('modeled','observed E 45 dregrees from a',2)
subplot(4,1,3); plot(v3,r_omega3_final,v3,r_omega3,'LineWidth',3.0);
xlim([400 2000]);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('modeled','observed E 90 dregrees from a',2)
subplot(4,1,4); plot(v4,r_omega4_final,v4,r_omega4,'LineWidth',3.0);
xlim([400 2000]);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('modeled','observed E 135 dregrees from a',2)
xlabel('Wavenumber (cm^{-1})','FontSize',14)
hold on;
hold off;

% plot initial guess and observed reflectance on the same axes
figure(2); clf;
subplot(4,1,1); plot(v1,r_omega1_guess,v1,r_omega1,'LineWidth',3.0);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('initial guess','observed E || a',2)
subplot(4,1,2); plot(v2,r_omega2_guess,v2,r_omega2,'LineWidth',3.0);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('initial guess','observed E 45 dregrees from a',2)
subplot(4,1,3); plot(v3,r_omega3_guess,v3,r_omega3,'LineWidth',3.0);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('initial guess','observed E 90 dregrees from a',2)
subplot(4,1,4); plot(v4,r_omega4_guess,v4,r_omega4,'LineWidth',3.0);
ylim([0 1]);
set(gca,'XDir','reverse')
ylabel('Reflectance')
legend('initial guess','observed E 135 dregrees from a',2)
xlabel('Wavenumber (cm^{-1})','FontSize',14)
hold on;
hold off;

% plot residual difference between the modeled and measured reflectance
figure(3); clf;
subplot(4,1,1); plot(v1,r_omega1-r_omega1_final,'-k','LineWidth',3.0);
xlim([400 2000]);
ylim([-0.5 0.5]);
set(gca,'XDir','reverse')
ylabel('Residual')
legend('E || a',2)
subplot(4,1,2); plot(v2,r_omega2-r_omega2_final,'-k','LineWidth',3.0);
xlim([400 2000]);
ylim([-0.5 0.5]);
set(gca,'XDir','reverse')
ylabel('Residual')
legend('E 45 degrees from a',2)
subplot(4,1,3); plot(v3,r_omega3-r_omega3_final,'-k','LineWidth',3.0);
xlim([400 2000]);
ylim([-0.5 0.5]);
set(gca,'XDir','reverse')
ylabel('Residual')
legend('E 90 degrees from a',2)
subplot(4,1,4); plot(v4,r_omega4-r_omega4_final,'-k','LineWidth',3.0);
xlim([400 2000]);
ylim([-0.5 0.5]);
set(gca,'XDir','reverse')
ylabel('Residual')
legend('E 135 degrees from a',2)
xlabel('Wavenumber (cm^{-1})','FontSize',14)

% plot modeled n and k
figure(4); clf;
subplot(2,1,1); plot(v1,n1,'g-',v1,n2,'b-','LineWidth',3.0);
xlim([400 2000]);
ylim([0 5]);
set(gca,'XDir','reverse')
ylabel('n1, n2');
xlabel('Wavenumber (cm^{-1})','FontSize',14);
legend('n1','n2',2);
subplot(2,1,2); semilogy(v1,abs(k1),'g-',v1,abs(k2),'b-','LineWidth',3.0);
xlim([400 2000]);
ylim([0 5]);
set(gca,'XDir','reverse')
ylabel('k1,k2');
xlabel('Wavenumber (cm^{-1})','FontSize',14);
legend('k1','k2',2);

%plot modeled reflectance curves w/ 95% ci
set(0,'DefaultAxesColorOrder',[1 0 0;.5 .5 0;0 1 0;0 .5 .5;0 0 1;.5 0 .5])
figure(5); clf;
subplot(5,1,1); plot(v1,r_omega1_upper,'g--',v1,r_omega1_final,'b-',v1,r_omega1_lower,'g--','Linewidth',3.0)
set(gca,'XDir','reverse');
legend('','modeled 0 degrees from a','');
subplot(5,1,2); plot(v2,r_omega2_upper,'g--',v2,r_omega2_final,'b-',v2,r_omega2_lower,'g--','Linewidth',3.0)
set(gca,'XDir','reverse');
legend('','modeled 45 degrees from a','');
subplot(5,1,3); plot(v3,r_omega3_upper,'g--',v3,r_omega3_final,'b-',v3,r_omega3_lower,'g--','Linewidth',3.0)
set(gca,'XDir','reverse');
legend('','modeled 90 degrees from a','');
subplot(5,1,4); plot(v4,r_omega4_upper,'g--',v4,r_omega4_final,'b-',v4,r_omega4_lower,'g--','Linewidth',3.0)
set(gca,'XDir','reverse');
legend('','modeled 135 degrees from a','');
%plot modeled reflectance curves for all orientations on the same axes
subplot(5,1,5); plot(v1,r_omega1_final,v2,r_omega2_final,v3,r_omega3_final,v4,r_omega4_final,'Linewidth',3.0);
set(gca,'XDir','reverse');
ylabel('Reflectance');
xlabel('Wavenumber (cm^{-1})','FontSize',14);
legend('0','45','90','135');
